@@ -27,7 +27,7 @@ THE SOFTWARE.
 #include <errno.h>
 
 #include "babeld.h"
-#include "network.h"
+#include "interface.h"
 #include "source.h"
 #include "neighbour.h"
 #include "xroute.h"
@@ -140,7 +140,7 @@ local_notify_neighbour(struct neighbour *neigh, int kind)
                      address as a unique identifier. */
                   (unsigned long int)neigh,
                   format_address(neigh->address),
-                  neigh->network->ifname,
+                  neigh->ifp->name,
                   neigh->reach,
                   neighbour_rxcost(neigh),
                   neighbour_txcost(neigh),
@@ -207,7 +207,7 @@ local_notify_route(struct route *route, int kind)
                   format_eui64(route->src->id),
                   route_metric(route), route->refmetric,
                   format_address(route->neigh->address),
-                  route->neigh->network->ifname);
+                  route->neigh->ifp->name);
     
     if(rc < 0 || rc >= 512)
         goto fail;
@@ -222,10 +222,22 @@ local_notify_route(struct route *route, int kind)
     return;
 }
 
-void
-local_dump()
+static void
+local_notify_xroute_callback(struct xroute *xroute, void *closure)
 {
-    int i, rc;
+    local_notify_xroute(xroute, LOCAL_ADD);
+}
+
+static void
+local_notify_route_callback(struct route *route, void *closure)
+{
+    local_notify_route(route, LOCAL_ADD);
+}
+
+void
+local_notify_all()
+{
+    int rc;
     struct neighbour *neigh;
     const char *header = "BABEL 0.0\n";
 
@@ -240,10 +252,8 @@ local_dump()
     FOR_ALL_NEIGHBOURS(neigh) {
         local_notify_neighbour(neigh, LOCAL_ADD);
     }
-    for(i = 0; i < numxroutes; i++)
-        local_notify_xroute(&xroutes[i], LOCAL_ADD);
-    for(i = 0; i < numroutes; i++)
-        local_notify_route(&routes[i], LOCAL_ADD);
+    for_all_xroutes(local_notify_xroute_callback, NULL);
+    for_all_routes(local_notify_route_callback, NULL);
     return;
 
  fail:
