@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2007, 2008 by Juliusz Chroboczek
+Copyright (c) 2007-2011 by Juliusz Chroboczek
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -39,10 +39,10 @@ struct route {
     unsigned short hold_time;    /* in seconds */
     short installed;
     unsigned char channels[DIVERSITY_HOPS];
+    struct route *next;
 };
 
-extern struct route *routes;
-extern int numroutes, maxroutes;
+extern struct route **routes;
 extern int kernel_metric, allow_duplicates;
 extern int diversity_kind, diversity_factor;
 extern int keep_unfeasible;
@@ -69,16 +69,20 @@ struct route *find_route(const unsigned char *prefix, unsigned char plen,
                          struct neighbour *neigh, const unsigned char *nexthop);
 struct route *find_installed_route(const unsigned char *prefix,
                                    unsigned char plen);
+int installed_routes_estimate(void);
 void flush_route(struct route *route);
+void flush_all_routes(void);
 void flush_neighbour_routes(struct neighbour *neigh);
-void flush_network_routes(struct network *net, int v4only);
+void flush_interface_routes(struct interface *ifp, int v4only);
+void for_all_routes(void (*f)(struct route*, void*), void *closure);
+void for_all_installed_routes(void (*f)(struct route*, void*), void *closure);
 void install_route(struct route *route);
 void uninstall_route(struct route *route);
 void switch_route(struct route *old, struct route *new);
 int route_feasible(struct route *route);
 int route_old(struct route *route);
 int route_expired(struct route *route);
-int route_interferes(struct route *route, struct network *net);
+int route_interferes(struct route *route, struct interface *ifp);
 int update_feasible(struct source *src,
                     unsigned short seqno, unsigned short refmetric);
 struct route *find_best_route(const unsigned char *prefix, unsigned char plen,
@@ -86,10 +90,10 @@ struct route *find_best_route(const unsigned char *prefix, unsigned char plen,
 struct route *install_best_route(const unsigned char prefix[16],
                                  unsigned char plen);
 void update_neighbour_metric(struct neighbour *neigh, int changed);
-void update_network_metric(struct network *net);
+void update_interface_metric(struct interface *ifp);
 void update_route_metric(struct route *route);
-struct route *update_route(const unsigned char *a,
-                           const unsigned char *p, unsigned char plen,
+struct route *update_route(const unsigned char *id,
+                           const unsigned char *prefix, unsigned char plen,
                            unsigned short seqno, unsigned short refmetric,
                            unsigned short interval, struct neighbour *neigh,
                            const unsigned char *nexthop,
