@@ -36,12 +36,11 @@ THE SOFTWARE.
 #include "xroute.h"
 #include "util.h"
 #include "configuration.h"
-#include "network.h"
+#include "interface.h"
 #include "local.h"
 
-struct xroute *xroutes;
-int numxroutes = 0;
-int maxxroutes = 0;
+static struct xroute *xroutes;
+static int numxroutes = 0, maxxroutes = 0;
 
 struct xroute *
 find_xroute(const unsigned char *prefix, unsigned char plen)
@@ -120,6 +119,22 @@ add_xroute(unsigned char prefix[16], unsigned char plen,
     return 1;
 }
 
+/* Returns an overestimate of the number of xroutes. */
+int
+xroutes_estimate()
+{
+    return numxroutes;
+}
+
+void
+for_all_xroutes(void (*f)(struct xroute*, void*), void *closure)
+{
+    int i;
+
+    for(i = 0; i < numxroutes; i++)
+        (*f)(&xroutes[i], closure);
+}
+
 int
 check_xroutes(int send_updates)
 {
